<?php

namespace Doctrine\Tests\Common\Annotations\Fixtures;

use
    Doctrine\Tests\Common\Annotations\Fixtures\Annotation\Route,
    Doctrine\Tests\Common\Annotations\Fixtures\Annotation\Secure;

class MultipleImportsInUseStatement
{
}
