<?php

declare(strict_types=1);

require_once __DIR__ . '/../../../../vendor/autoload.php';
require_once __DIR__ . '/../../Tests/Common/Annotations/Fixtures/Annotation/Route.php';
require_once __DIR__ . '/../../Tests/Common/Annotations/Fixtures/Annotation/Secure.php';
require_once __DIR__ . '/../../Tests/Common/Annotations/Fixtures/Annotation/Template.php';
