<?php

class SingleClassLOC1000
{
    public const TEST1 = 1234567890;
    public const TEST2 = 1234567890;
    public const TEST3 = 1234567890;
    public const TEST4 = 1234567890;
    public const TEST5 = 1234567890;
    public const TEST6 = 1234567890;
    public const TEST7 = 1234567890;
    public const TEST8 = 1234567890;
    public const TEST9 = 1234567890;

    /** @var null */
    private $test1 = null;
    /** @var null */
    private $test2 = null;
    /** @var null */
    private $test3 = null;
    /** @var null */
    private $test4 = null;
    /** @var null */
    private $test5 = null;
    /** @var null */
    private $test6 = null;
    /** @var null */
    private $test7 = null;
    /** @var null */
    private $test8 = null;
    /** @var null */
    private $test9 = null;

    public function test1(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test2(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test3(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test4(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test5(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test6(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test7(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test8(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test9(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test10(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test11(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test12(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test13(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test14(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test15(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test16(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test17(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test18(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test19(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test20(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test21(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test22(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test23(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test24(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test25(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test26(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test27(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test28(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test29(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test30(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test31(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test32(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test33(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test34(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test35(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test36(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test37(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test38(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }

    public function test39(): string
    {
        echo $this->test1;
        echo $this->test2;
        echo $this->test3;
        $array = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
        foreach ($array as $key => $value) {
            echo $key . ' => ' . $value;
        }

        $val  = (string) self::TEST1;
        $val .= (string) self::TEST2;
        $val .= (string) self::TEST3;
        $val .= (string) self::TEST4;
        $val .= (string) self::TEST5;
        $val .= (string) self::TEST6;
        $val .= (string) self::TEST7;
        $val .= (string) self::TEST8;
        $val .= (string) self::TEST9;

        strtolower($val);

        return $val;
    }
}
