<?php

namespace Doctrine\Tests\Common\Annotations\Fixtures;

use
    \Doctrine\Tests\Common\Annotations\Fixtures\Annotation\Secure,
    \Doctrine\Tests\Common\Annotations\Fixtures\Annotation\Route;
use \Doctrine\Tests\Common\Annotations\Fixtures\Annotation\Template;

class ClassWithFullyQualifiedUseStatements
{
}
